############################################################
#                                                          #
#              Information                                 #
#                                                          #
############################################################

# Dates: See config File. We dont have data for 31 Dec. and the code don't resist it. We need more robustness   
# Time zone and hour format: Hour of the day is shown in UTC. Value at Hour 1 correspond to the hour starting at 1 and ending at 2. As April falls in summer time (UTC + 2 hours), to convert from local time to UTC we move two hours backwards. So, 7 am local time is 5 am UTC.   
# Pollutant: Depends on the variable pollutant in config_file   
# Catalunya    
# Sites: monitoring stations, the number of them depends on seet_stations code
# Observations source: 
# Models intercompared:  
# Raw model from Caliope
# LOOCV applied to Caliope
#-------------------------------------------------------------------------------
# Measure the time it takes to run the analysis
start_time <- Sys.time()
#-------------------------------------------------------------------------------
#              UK / LOOCV METHODOLOGY 
#-------------------------------------------------------------------------------

############################################################
#                                                          #
#              Set libraries                               #
#                                                          #
############################################################
# ! check the version of the package or R if any library or function does not work correctly

library(urbanR)
library(RNetCDF)
library(sp)
library(sf)
#library(mapview)
library(ncdf4)
library(rgeos) 
library(ggplot2)
library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
library(dplyr) 
library(broom)
library(magrittr)
library(ggpubr)
library(raster)
library(rgdal)
#library(tmap)
library(gstat)
library(sp)
library(abind)
library(openair)
#library(Metrics)

############################################################
#                                                          #
#              Load setting                                #
#                                                          #
############################################################
## LOAD config_file
args = commandArgs(TRUE)
if (length(args) > 0) {
  configFile_path <- args[1]
} else {
  cat("\n***WARNING: no ConfigFile was provided. Using default ConfigFile.\n")
}
#config_file it's a file where we change the data, where we want to save the results, etc. so now we extract these data from it:
#configFile_path <- "/esarchive/scratch/mhajji/codes/geostats/config_file.yml"
#configFile_path <- "/esarchive/scratch/jmateu/Codes/run_geostats/config_file.yml"

config          <- yaml::yaml.load_file(configFile_path)
sim_folder_path <- config$sim_folder_path
obs_path        <- config$obs_path
model_path      <- config$model_path
stations_path   <- config$stations_path

pollutant       <- config$pollutant 
print(paste0("Selected pollutant: ", pollutant))

time_resolution <- config$time_resolution # hourly or daily
file_format     <- config$file_format # ghost or gene

if(time_resolution=="hourly"){
  print("The model and observation are in hourly format")
} else if(time_resolution=="daily"){
  print("The model and observations are in daily format")
} else{
  stop("Wrong time_resolution. Please check the configuration.")
}

date_time_end   <- config$date_time_end
date_time_begin <- config$date_time_begin

name            <- config$doc_name
variogram_path  <- config$variogram_path
path_code       <- config$path_code
burnes_path     <- config$burnes_path

set_stations       <- config$set_stations
set_observations   <- config$set_observations 
set_model          <- config$set_model
execute_loocv      <- config$execute_loocv
execute_statistics <- config$execute_statistics
execute_ok         <- config$execute_ok

compute_in_UTM         <- config$compute_in_UTM
crs_model              <- config$crs_model
crs_model_utm          <- config$crs_model_utm

save_result                <- config$save_result
save_model                 <- config$save_model
save_obs                   <- config$save_obs
save_OK                    <- config$save_OK
save_LOOCV                 <- config$save_LOOCV
save_stations              <- config$save_stations

save_general_statistics    <- config$save_general_statistics 
save_time_series           <- config$save_time_series 
save_temporal_variation    <- config$save_temporal_variation 

# As we had problems with Ordinary and Universal Kriging variograms, we saved one of each to read them in case we have problems with the one executed for a specific date or hour
#NOTE: in principle, if it was from the gene or ghost, it would not affect the variogram, rather than the contaminant itself. But in this case, we will save one for each.
variogram_ok <- paste0(variogram_path, "Variogram_OK_",file_format,"_",pollutant,".rds") 
variogram_uk <- paste0(variogram_path, "Variogram_UK_",file_format,"_",pollutant,".rds")

# Set Units
print("Setting units of obs and model")
# We need to adjust the units of the observations to ug/m3, so first we pick up the constant depending on the pollutant that we are working with
pol          <- c("no2","o3","pm10","pm25")
k            <- c(1.881, 1.963, 1, 1) # Based on obs constant
# We will use the limit parameter to avoid observations that we consider wrong in loocv loop. As for NO2 the alert is established at 400 ug/m3, and 240 for O3 we will put this number as the boundary

############################################################
#           !!!!   WARNING!!!!                             #
############################################################
#!!!!!!!!!!! Mirar tema de les Flags:
limit        <- c(400, 240, 100, 100) #the limit for pm10 is 50 ug/m3, 25 ug/m3, but i put 100 just in case 

position     <- match(pollutant, pol) 
number       <- match(pol[position], pol)

limit        <- limit[number]
obs_constant <- k[number]

#to arrange set_stations or observations, we need to configure name_pollutant, based in how the script name with the data is saved
if(pollutant %in% c("no2","o3")){
  name_pollutant   <- paste0("sconc",pollutant)
  mod_constant     <- k[number]*1e03
  min_obs          <- 20 #min nº of obs that we consider to apply uk and loocv
  
} else{
  #at the moment for pm10 and pm2.5
  name_pollutant  <- pollutant
  mod_constant    <- k[number]*1e09 #in the script of pm the units are kg/m3 but we are sure that they are worng, and they are in yg/m3
  min_obs         <- 5 #min nº of obs that we consider to apply uk and loocv
}

############################################################
#                                                          #
#              Set temporal context                        #
#                                                          #
############################################################
print("setting dates")
#date_times et fa un super vector d'una  columna, dia per dia, hora per hora des de l'inici fins a la data final p.ex: 2021-01-01 00:00:00
date_times <- data_frame_dates(date_time_begin = date_time_begin,
                               date_time_end = date_time_end)

#dates: Llista de caracters anymesdia p.ex: 20210101
dates <- unique(paste0(as.character(strftime(date_times[, 1], format="%Y", tz = "UTC")),
                       as.character(strftime(date_times[, 1], format="%m", tz = "UTC")),
                       as.character(strftime(date_times[, 1], format="%d", tz = "UTC"))))
#year_month: igual que date pero anymes p.ex: 202101
year_month <- unique(paste0(as.character(strftime(date_times[, 1], format="%Y", tz = "UTC")),
                            as.character(strftime(date_times[, 1], format="%m", tz = "UTC"))))

#For csv files, the name of obs files are referred to the year, so, at the moment we pick the first year in dates list:
#read_year <- substr(dates, 1, 4)[1]
read_year <- unique(substr(dates, 1, 4)) #with this command we will read the years in dates

############################################################
#                                                          #
#                 Load Observations                        #
#                                                          #
############################################################
## To do: make sure df_obs_all is exactly equal for gene/GHOST so the subsetting has no if's
print("Load observations")

if (set_observations){
  source(paste0(path_code,"load_obs_cat.R"))
  print(paste0("Observations load for ",pollutant, " using data from ", file_format, ": done"))
  print("df_obs_all computed")
  
} else {
  df_obs_all <- readRDS(file = paste0(save_result,"df_obs_",file_format,"_",pollutant,".rds"))
  print("df_obs_all loaded from previous loading")
} 

# To subset the data:
if(file_format=="ghost"){
  if(time_resolution=="hourly"){
    df_obs_sub <- time_subset(data = df_obs_all, time_var_name = "date",
                              date_time_begin = date_time_begin,
                              date_time_end = date_time_end)
  } else { 
    print("ERROR THE CODE IS NOT FINISHED YET") #de moment no fem servir mitjanes diàries de ghost (mirar)
  }
  
  } else if (file_format=="gene") {
    if(time_resolution=="hourly"){
    start_date  <- format(head(date_times$date, 1), format="%Y-%m-%d %H:%M:%S")
    end_date    <- format(tail(date_times$date, 1), format="%Y-%m-%d %H:%M:%S")
    df_obs_sub  <- subset(df_obs_all, date>=start_date & date <= end_date)
    
    } else if(time_resolution=="daily"){ 
    start_date  <- as.Date(as.character(dates[1]), format="%Y%m%d")
    end_date    <- as.Date(as.character(tail(dates,n=1)), format="%Y%m%d")
    df_obs_sub  <- subset(df_obs_all, date>=start_date & date <= end_date)
    }else{
    print("wrong time_resolution")
  }
  }
print("Read observations: done")

############################################################
#                                                          #
#                 Load Local Stations                      #
#                                                          #
############################################################
print("Load Local Stations")
# local_stations: special points data frame with two cols. For example, for NO2 pollutant [NO2 and ref_obs-reference of station-], at the moment we have 72 stations in Cat for this pollutant 

if (set_stations){
  source(paste0(path_code,"set_stations.R"))
  print(paste0("Set stations load for ",pollutant, " using data from ", file_format, ": done"))
} else {
  local_stations <- readRDS(file = paste0(save_result,"local_stations_",file_format, "_", pollutant,".rds"))
  print(paste0("Read local stations for ",pollutant, " using data from ", file_format, ": done"))
}

############################################################
#                                                          #
#                 Load Model                               #
#                                                          #
############################################################
print("Load Model")
if (set_model){
  source(paste0(path_code,"load_caliope_cat.R")) 
  print("Load Raw model load: done")
} else {
  #Not working: model <- readRDS(file = paste0(save_result,"model_",file_format, "_",pollutant,".rds"))
  model <- readRDS(file = paste0(save_result,"model_",pollutant,".rds"))
  print("Preload raw model: done")
}


############################################################
#                                                          #
#                 Interpolate model to AQMS                #
#                                                          #
############################################################
# Input : date_times, local_stations, local_stations, df_obs_sub, model
# Output: llista_uk_total_df  


#idem read obs
print("Loop Universal Kriging")
if (execute_ok){
  source(paste0(path_code,"OK_interpolate_model_to_AQMS.R"))  
  print("Interpolate model to AQMS: computed")
} else {
  llista_uk_total_df <- readRDS(paste0(save_result,"llista_uk_",pollutant,"_df.rds"))
  print("Interpolate model to AQMS: loaded from pervious files")
}

############################################################
#                                                          #
#                 Loop Universal Kriging                   #
#                                                          #
############################################################

#idem read obs
print("Loop Universal Kriging")
if (execute_loocv){
  source(paste0(path_code,"loop_uk_cat.R"))  
  print("Corrected model load: done")
} else {
  # Jan: not working LOOCV_total     <- readRDS(file = paste0(save_result,"LOOCV_",file_format, "_",pollutant,".rds"))
  # Jan: not working  LOOCV_total_df     <- readRDS(file = paste0(save_result,"LOOCV_",file_format, "_",pollutant,"_df.rds"))
  LOOCV_total     <- readRDS(file=paste0(save_result,"LOOCV_",pollutant,".rds"))
  LOOCV_total_df  <- readRDS(file=paste0(save_result,"LOOCV_",pollutant,"_df.rds")) 
  #llista_uk_total_df <- readRDS(file = paste0(save_result,"llista_uk_",file_format, "_",pollutant,"_df.rds"))
  print("Read corrected model: done")
}

############################################################
#                                                          #
#              evaluation statistics                       #
#                                                          #
############################################################

print("Evaluating statistics")

if (execute_statistics){
  
  # Evaluation stadistics
  source(paste0(path_code,"stadistics_uk_cat.R")) 
  print("Evaluation statistics LOOCV vs raw model: done")
  
  #Temporal Variation
  source(paste0(path_code,"temporal_variation_uk_cat.R")) 
  print("Temporal variation: done")
  
  #Time Series
  source(paste0(path_code,"time_series_uk_cat.R")) 
  print("Time Series avaluation: done")
  
  
} else {
  print("Evaluation Statistics not executed")
}

#-------------------------------------------------------------------------------
#Temps que triga en executar-se el fitxer

end_time <- Sys.time()
temps_total <- end_time-start_time
print(temps_total)
